﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using System.IO;
using System.IO.Compression;
using System.Collections.ObjectModel;
using System.Diagnostics;

namespace GeoDbMetaDataLimit
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }


        private void Form1_Load(object sender, EventArgs e)
        {
            listView1.View = View.Details;

            listView1.Columns.Add("Column", 100);
            listView1.Columns.Add("Value", 100);

            listView2.View = View.Details;

            listView2.Columns.Add("Column", 100);
            listView2.Columns.Add("Value", 100);

        }

        private void btnCreateDb_Click(object sender, EventArgs e)
        {

            // Create a db with 100 columns of metadata
            string db = @"..\..\Data\test.gdb";

            if (new DirectoryInfo(db).Exists)
            {
                Directory.Delete(db, true);
            }

            FileGeoDatabaseFeatureLayer.CreateFileGeoDatabase(db);

            Collection<FeatureSourceColumn> columns = new Collection<FeatureSourceColumn>();

            //for (int i = 0; i < 100; i++)
            //{
            //    columns.Add(new FeatureSourceColumn(string.Format("Column_{0}", i)));
            //}

            string filePath = @"..\..\Data\MonkeyPic.JPG";
            //A stream of bytes that represnts the binary file 
            FileStream fs = new FileStream(filePath, FileMode.Open, FileAccess.Read);

            //The reader reads the binary data from the file stream 
            BinaryReader reader = new BinaryReader(fs);
            //Bytes from the binary reader stored in BlobValue array 
            byte[] bytes = new byte[(int)fs.Length];
            fs.Read(bytes, 0, (int)fs.Length);
            fs.Close();
            string blobString = Convert.ToBase64String(bytes);

            FeatureSourceColumn blobColumn = new FeatureSourceColumn("Blob", "blob", 1);
            FeatureSourceColumn intColumn = new FeatureSourceColumn("Integer", "integer", 1);
            FeatureSourceColumn doubleColumn = new FeatureSourceColumn("Double", "double", 0);
            FeatureSourceColumn datColumn = new FeatureSourceColumn("Date", "datetime", 0);

            columns.Add(blobColumn);
            columns.Add(intColumn);
            columns.Add(doubleColumn);
            columns.Add(datColumn);

            FileGeoDatabaseFeatureLayer.CreateTable(db, "MY_TABLE", WellKnownType.Point, columns);

            //Now add a record to it
            Dictionary<string, string> columnsToAdd = new Dictionary<string, string>();

            // These types are working well
            columnsToAdd.Add("Blob", blobString);
            columnsToAdd.Add("Integer", "10000");
            columnsToAdd.Add("Double", "1.3333");

            // but Date always seems to come out as text in the database
            columnsToAdd.Add("Date", "10/02/1969");

            FileGeoDatabaseFeatureLayer fgdb = new FileGeoDatabaseFeatureLayer(db, "MY_TABLE");

            for (int i = 0; i < 50; i++)
            {
                fgdb.Open();
                fgdb.FeatureSource.BeginTransaction();

                Feature featureToAdd = new Feature(new PointShape(0, i * 10), columnsToAdd);
                fgdb.FeatureSource.AddFeature(featureToAdd);
                
                fgdb.FeatureSource.CommitTransaction();
                fgdb.Close();
            }

            MessageBox.Show("Database Created");
        }

        private void btnCreateDb_ClickOld(object sender, EventArgs e)
        {

            // Create a db with 100 columns of metadata
            string db = @"..\..\Data\test.gdb";

            if (new DirectoryInfo(db).Exists)
            {
                Directory.Delete(db, true);
            }

            FileGeoDatabaseFeatureLayer.CreateFileGeoDatabase(db);

            Collection<FeatureSourceColumn> columns = new Collection<FeatureSourceColumn>();

            //for (int i = 0; i < 100; i++)
            //{
            //    columns.Add(new FeatureSourceColumn(string.Format("Column_{0}", i)));
            //}

            //string filePath = @"..\..\Data\MonkeyPic.JPG";
            string filePath = @"..\..\Data\Monkey.pdf";
            //string zipFile = @"..\..\Data\MonkeyPic.gz";
            //A stream of bytes that represnts the binary file 
            FileStream fs = new FileStream(filePath, FileMode.Open, FileAccess.Read);

            //FileStream compressedStream = File.Create(zipFile);

            //using (GZipStream gzip = new GZipStream(compressedStream, CompressionMode.Compress))
            //{
            //    fs.CopyTo(compressedStream);
            //}

            //FileStream gzipStream = new FileStream(zipFile, FileMode.Open, FileAccess.Read);

            //The reader reads the binary data from the file stream 
            //BinaryReader reader = new BinaryReader(gzipStream);
            BinaryReader reader = new BinaryReader(fs);
            //Bytes from the binary reader stored in BlobValue array 
            byte[] blobValue = reader.ReadBytes((int)fs.Length);
            string blobString = Convert.ToBase64String(blobValue);

            FeatureSourceColumn blobColumn = new FeatureSourceColumn("Blob", "blob", 1);
            FeatureSourceColumn intColumn = new FeatureSourceColumn("Integer", "integer", 1);
            FeatureSourceColumn doubleColumn = new FeatureSourceColumn("Double", "double", 0);
            FeatureSourceColumn datColumn = new FeatureSourceColumn("Date", "datetime", 0);
            
            columns.Add(blobColumn);
            columns.Add(intColumn);
            columns.Add(doubleColumn);
            columns.Add(datColumn);

            FileGeoDatabaseFeatureLayer.CreateTable(db, "MY_TABLE", WellKnownType.Point, columns);

            //Now add a record to it
            Dictionary<string, string> columnsToAdd = new Dictionary<string, string>();
            
            // These types are working well
            columnsToAdd.Add("Blob", blobString);
            columnsToAdd.Add("Integer", "10000");
            columnsToAdd.Add("Double", "1.3333");
            
            // but Date always seems to come out as text in the database
            columnsToAdd.Add("Date", "10/02/1969");

            FileGeoDatabaseFeatureLayer fgdb = new FileGeoDatabaseFeatureLayer(db, "MY_TABLE");

            for (int i = 0; i < 5; i++)
            {
                fgdb.Open();
                fgdb.FeatureSource.BeginTransaction();

                Feature featureToAdd = new Feature(new PointShape(0, i * 10), columnsToAdd);
                fgdb.FeatureSource.AddFeature(featureToAdd);

                fgdb.FeatureSource.CommitTransaction();
                fgdb.Close();
            }

            MessageBox.Show("Database Created");
        }

        private void btnReadDb_Click(object sender, EventArgs e)
        {
            // Read back the table and populate the list box

            string db = @"..\..\Data\test.gdb";
            //string db = @"..\..\Data\Export_Output.gdb";

            Collection<string> allTables = FileGeoDatabaseFeatureLayer.GetTableNames(db);

            FileGeoDatabaseFeatureLayer fgdb = new FileGeoDatabaseFeatureLayer(db, allTables[0]);

            listView1.Items.Clear();

            fgdb.Open();

            // Populate List 1 with the attributes straight from the feature class
            foreach (Feature feature in fgdb.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns))
            {
                // Add all the columns to the list
                foreach (KeyValuePair<string, string> item in feature.ColumnValues)
                {
                    ListViewItem itemToAdd = new ListViewItem();
                    itemToAdd.Text = item.Key;
                    itemToAdd.SubItems.Add(item.Value);
                    listView1.Items.Add(itemToAdd);
                }
            }

            // Now Populate list 2 with all the columns using the featuresource Get Columns
            foreach (Feature feature in fgdb.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns))
            {
                foreach (FeatureSourceColumn column in fgdb.FeatureSource.GetColumns())
                {
                    ListViewItem itemToAdd = new ListViewItem();
                    itemToAdd.Text = column.ColumnName;
                    itemToAdd.SubItems.Add(feature.ColumnValues[column.ColumnName]);
                    listView2.Items.Add(itemToAdd);
                }
            }

            fgdb.Close();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            string db = @"..\..\Data\test.gdb";
            FileGeoDatabaseFeatureLayer fgdb = new FileGeoDatabaseFeatureLayer(db, "MY_TABLE");

            fgdb.Open();

            Feature f = fgdb.FeatureSource.GetFeatureById("1", ReturningColumnsType.AllColumns);

            byte[] tempBytes = Convert.FromBase64String(f.ColumnValues["Blob"]);

            string outFile = System.IO.Path.GetTempFileName();
            FileStream outStream = File.Create(outFile, tempBytes.Length);

            outStream.Write(tempBytes, 0, tempBytes.Length);

            // Show an image in a picture box
            //pictureBox1.Image = System.Drawing.Image.FromStream(outStream);

            // And launch the file in the default viewer
            //ProcessStartInfo startInfo = new ProcessStartInfo();
            //startInfo.FileName = "mspaint.exe";
            //startInfo.Arguments = outFile;
            //Process.Start(startInfo);


            outStream.Close();
            File.Delete(outFile);

            

            fgdb.Close();
        }



    }
}
